@@ -1,9 +1,13 @@
+import datetime
+import pytz
 import boto3
 import logging
 from time import sleep
-from quickbe import Log
+from autom8it import Log
 from autom8it import AutomationTask
 
+UTC = pytz.UTC
+
 for log_name in ['boto', 'boto3', 'botocore', 's3transfer', 'urllib3']:
     logging.getLogger(log_name).setLevel(logging.WARNING)
 
@@ -200,7 +204,7 @@ class VerifyECSServicesCount(AutomationTask):
             },
         }
 
-    aws_client = boto3.client('ecs')
+    ecs_client = boto3.client('ecs')
 
     def __init__(self, task_data: dict):
         super().__init__(task_data=task_data)
@@ -209,14 +213,17 @@ class VerifyECSServicesCount(AutomationTask):
     def task_type(self) -> str:
         return 'Verify ECS services count'
 
+    def describe_services(self) -> dict:
+        return self.ecs_client.describe_services(
+            cluster=self.get_task_attribute(self.CLUSTER_KEY),
+            services=self.get_task_attribute(self.SERVICES_KEY),
+        )
+
     def do(self):
         return 'OK'
 
     def is_done(self) -> bool:
-        desc = self.aws_client.describe_services(
-            cluster=self.get_task_attribute(self.CLUSTER_KEY),
-            services=self.get_task_attribute(self.SERVICES_KEY),
-        )
+        desc = self.describe_services()
         for service in desc[self.SERVICES_KEY]:
             desired_count = service[self.SERVICE_DESIRED_COUNT_KEY]
             running_count = service[self.SERVICE_RUNNING_COUNT_KEY]
@@ -242,7 +249,7 @@ class UpdateServicesWithLastRevision(VerifyECSServicesCount):
         result = []
         for service in self.get_task_attribute(self.SERVICES_KEY):
 
-            desc = self.aws_client.update_service(
+            desc = self.ecs_client.update_service(
                 cluster=self.get_task_attribute(self.CLUSTER_KEY),
                 service=service,
                 taskDefinition=self._get_task_definition_family(service=service)
@@ -252,7 +259,7 @@ class UpdateServicesWithLastRevision(VerifyECSServicesCount):
         return result
 
     def _get_task_definition_family(self, service: str) -> str:
-        desc = self.aws_client.describe_services(
+        desc = self.ecs_client.describe_services(
             cluster=self.get_task_attribute(self.CLUSTER_KEY),
             services=[service]
         )
@@ -279,14 +286,11 @@ class ChangeTaskDefinitionContainerImage(AutomationTask):
             self.TASK_DEFINITION_KEY: {
                 'required': True,
                 'type': 'string'
-            },
-            self.IMAGE_KEY: {
-                'required': True,
-                'type': 'string'
-            },
+            }
         }
 
-    aws_client = boto3.client('ecs')
+    aws_ecs_client = boto3.client('ecs')
+    aws_ecr_client = boto3.client('ecr')
 
     def __init__(self, task_data: dict):
         super().__init__(task_data=task_data)
@@ -296,20 +300,32 @@ class ChangeTaskDefinitionContainerImage(AutomationTask):
         return 'Create ECS Task Definition revision'
 
     def do(self):
-        desc = self.aws_client.describe_task_definition(
+        desc = self.aws_ecs_client.describe_task_definition(
             taskDefinition=self.get_task_attribute(self.TASK_DEFINITION_KEY)
         )
         Log.debug(f'Task definition description: {desc}')
         container_definitions = desc['taskDefinition'][self.CONTAINER_DEFINITIONS_KEY]
         for container_def in container_definitions:
-            image_uri = self.get_task_attribute(self.IMAGE_KEY)
+            image_uri = self.get_task_attribute(self.IMAGE_KEY, default=None)
+            if image_uri is None:
+                image_uri = self.get_last_image_tag(
+                    repository=self.get_repository_name(image_uri=container_def[self.IMAGE_KEY])
+                )
             if ':' not in image_uri:
                 image_uri = f"{container_def[self.IMAGE_KEY].split(':')[0]}:{image_uri}"
             container_def[self.IMAGE_KEY] = image_uri
 
-        result = self.aws_client.register_task_definition(
+        task_definition_desc = desc.get('taskDefinition', {})
+
+        result = self.aws_ecs_client.register_task_definition(
             family=self.get_task_attribute(self.TASK_DEFINITION_KEY),
-            containerDefinitions=container_definitions
+            containerDefinitions=container_definitions,
+            taskRoleArn=task_definition_desc.get('taskRoleArn', ''),
+            executionRoleArn=task_definition_desc.get('executionRoleArn', ''),
+            networkMode=task_definition_desc.get('networkMode'),
+            volumes=task_definition_desc.get('volumes', []),
+            placementConstraints=task_definition_desc.get('placementConstraints', []),
+            requiresCompatibilities=task_definition_desc.get('requiresCompatibilities', [])
         )
 
         return result
@@ -317,3 +333,60 @@ class ChangeTaskDefinitionContainerImage(AutomationTask):
     def is_done(self) -> bool:
         return True
 
+    @staticmethod
+    def get_repository_name(image_uri: str) -> str:
+        """
+        Example: 112358132134.dkr.ecr.us-east-2.amazonaws.com/my-repo:1.10.21-tag
+        Will return my-repo
+        :param image_uri:
+        :return:
+        """
+        return image_uri.split('/')[1].split(':')[0]
+
+    @staticmethod
+    def get_last_image_tag(repository: str) -> str:
+        result = ChangeTaskDefinitionContainerImage.aws_ecr_client.describe_images(
+            repositoryName=repository
+        )
+        images = {image['imageTags'][0]: image['imagePushedAt'] for image in result['imageDetails']}
+        last_date = datetime.datetime(year=1970, month=1, day=1).replace(tzinfo=UTC)
+        last_tag = None
+        for tag, date in images.items():
+            date = date.replace(tzinfo=UTC)
+            if date > last_date:
+                last_date = date
+                last_tag = tag
+        return last_tag
+
+
+class StopServicesAndWaitUntilStarts(VerifyECSServicesCount):
+
+    def __init__(self, task_data: dict):
+        super().__init__(task_data=task_data)
+
+    @property
+    def task_type(self) -> str:
+        return 'Stop all task of ECS services wait until they starts again'
+
+    def stop_tasks(self, tasks_arn: list):
+        for task_arn in tasks_arn:
+            self.ecs_client.stop_task(
+                cluster=self.get_task_attribute(self.CLUSTER_KEY),
+                task=task_arn,
+                reason=f'Stopping service by Autom8it ({self.__class__.__name__})'
+            )
+
+    def do(self):
+        result = []
+        for service in self.get_task_attribute(self.SERVICES_KEY):
+
+            desc = self.ecs_client.list_tasks(
+                cluster=self.get_task_attribute(self.CLUSTER_KEY),
+                serviceName=service,
+                desiredStatus='RUNNING',
+            )
+            tasks_arn = desc.get('taskArns', [])
+            self.stop_tasks(tasks_arn=tasks_arn)
+            result.append(tasks_arn)
+
+        return result
